"""
The partition commands are used to partition edges, faces, and cells into new regions. A partition command 
can be invoked for a Part object or for an Assembly object. The partition commands create Feature objects 
and are described in Feature object.
"""

